package be.iminds.ilabt.jfed.experimenter_gui.dialogs;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.lowlevel.TestbedInfoSource;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.IntegerProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleIntegerProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.ListView;
import javafx.scene.control.cell.CheckBoxListCell;
import javafx.util.StringConverter;

import javax.inject.Inject;
import java.util.Comparator;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * User: twalcari
 * Date: 12/5/13
 * Time: 9:21 PM
 */
public class SelectAuthoritiesController {
    private final ObservableList<SelectableAuthority> selectableAuthorities = FXCollections.observableArrayList();

    private final IntegerProperty selectedAuthoritiesCount = new SimpleIntegerProperty();
    private final TestbedInfoSource testbedInfoSource;
    @FXML
    private ListView<SelectableAuthority> authoritiesListView;

    @Inject
    SelectAuthoritiesController(TestbedInfoSource testbedInfoSource) {
        this.testbedInfoSource = testbedInfoSource;
    }

    @FXML
    private void initialize() {
        selectableAuthorities.addAll(
                testbedInfoSource.getServers().stream()
                        .filter(s -> !s.isEdgeVlan())
                        .map(SelectableAuthority::new)
                        .sorted(Comparator.comparing(auth -> auth.getExactSfaAuthority().getName().toLowerCase()))
                        .collect(Collectors.toList()));

        authoritiesListView.setItems(selectableAuthorities);


        authoritiesListView.setCellFactory(
                CheckBoxListCell.forListView(SelectableAuthority::selectedProperty,
                        new StringConverter<SelectableAuthority>() {
                            @Override
                            public String toString(SelectableAuthority selectableAuthority) {
                                return selectableAuthority.getExactSfaAuthority().getName();
                            }

                            @Override
                            public SelectableAuthority fromString(String s) {
                                return null;
                            }
                        }));
    }

    private class SelectableAuthority {
        private final BooleanProperty selected = new SimpleBooleanProperty(false);
        private final Server sfaAuthority;

        private SelectableAuthority(Server sfaAuthority) {
            this.sfaAuthority = sfaAuthority;

            selected.addListener((observable, oldValue, newValue) -> {
                if (newValue != oldValue) {
                    if (newValue) {
                        selectedAuthoritiesCount.set(selectedAuthoritiesCount.get() + 1);
                    } else {
                        selectedAuthoritiesCount.set(selectedAuthoritiesCount.get() - 1);
                    }
                }
            });

        }

        public boolean getSelected() {
            return selected.get();
        }

        public void setSelected(boolean selected) {
            this.selected.set(selected);
        }

        public BooleanProperty selectedProperty() {
            return selected;
        }

        public Server getExactSfaAuthority() {
            return sfaAuthority;
        }
    }

    public int getSelectedAuthoritiesCount() {
        return selectedAuthoritiesCount.get();
    }

    public IntegerProperty selectedAuthoritiesCountProperty() {
        return selectedAuthoritiesCount;
    }

    public Set<Server>  getSelectedServers(){
        return selectableAuthorities.stream()
                .filter(SelectableAuthority::getSelected)
//                        .map(SelectableAuthority::getUsedSfaAuthority) caller has to worry about which auth to use, not us!
                .map(SelectableAuthority::getExactSfaAuthority)
                .collect(Collectors.toSet());
    }
}
